% M106 Winter 2021, Data-driven UQ
% by Yoonsang Lee (yoonsang.lee@dartmouth.edu)
%
% Lecture 8: Importance Sampling
%
% Calculate a rare event probability of the standard normal
% int_x^{\infty} normpdf(x_0)dx = 1-normCDF(x_0)
% using sampling methods
%
% Last update: Jan 20, 2021

clear

n=10000; % sample size
x_0 = 4.5; % extreme value

%% Using the Standard Monte Carlo
z_gauss = randn(n,1);
est_MC = sum(z_gauss>x_0)/n;

% Print the estimator
fprintf("\nTrue value, 1-normCDF(x_0): %.4e\n",1-normcdf(x_0))
fprintf("Using Monte Carlo Sampling: %.4e\n\n",est_MC)


%% Using Importance Sampling with a Cauchy distribution
z_cauchy = tan((rand(1,n) - 0.5)*pi);

cauchypdf = @(x) 1/pi./(1+x.^2);
w = normpdf(z_cauchy)./cauchypdf(z_cauchy);

est_IS=sum(w(z_cauchy>x_0))/n;

% Print the estimator
fprintf("\nTrue value, 1-normCDF(x_0): %.4e\n",1-normcdf(x_0))
fprintf("Using  Importance Sampling: %.4e\n\n",est_IS)
