# M106 Winter 2021, Data-driven UQ
# by Yoonsang Lee (yoonsang.lee@dartmouth.edu)
#
# Lecture 8: Importance Sampling
#
# Calculate a rare event probability of the standard normal
# int_x^{\infty} normpdf(x)dx = 1-normCDF(x)
# using sampling methods
#
# Last update: Jan 20, 2021

import numpy as np
from scipy.stats import norm, cauchy

n = 50000 # sample size
x_0 = 4.5

# Using the Standard Monte Carlo
z_gauss = np.random.randn(n,1)
est_MC = np.sum(z_gauss>x_0)/n;

# Using Importance Sampling with a Cauchy distribution
z_cauchy = np.tan((np.random.rand(n,1)-0.5)*np.pi)

w = norm.pdf(z_cauchy)/cauchy.pdf(z_cauchy)
est_IS = np.sum(w[z_cauchy>x_0])/n;

# Print the  estimator
print("True value, 1-normCDF(x_0): ",1-norm.cdf(x_0))
print("Using Monte Carlo Sampling: ",est_MC)
print("Using  Importance Sampling: ",est_IS)
